// MultiHack 1.2 header
// author: Jacob
// ------------------------------------------
// Include this in your project (along with multihack.lib) to make compatible with MultiHack.
// Once compiled, simply add the dll to multihack.cfg and load the game!


// DEFINES
// ------------------------------------------
	 #ifndef _MULTIHACK_H_
	 #define _MULTIHACK_H_
	 #include <d3dx9.h>
	 #include <windows.h>

typedef D3DCOLOR COLOR;


struct SCRIPT {	WORD OpCode; char Params[13]; };

// CLASSES (NEW in 1.2)
// ------------------------------------------
	struct _CSettings
	{
		int legendOffsetX;			// X position the legend is drawn on screen		  default: 10
		int legendOffsetFromY;		// Y offset from the bottom the legend is drawn   default: 7
		COLOR legendColorCMD;		// Color of the commands drawn in the legend	  default: green
		COLOR legendColorDESC;		// Color of the descriptions drawn in the legend  default: white
		int conOffsetX;				// X position the console is drawn on screen	  default: screenwidth / 2
		int conOffsetFromY;			// Y offset from the bottom the console is drawn  default: 7
		bool bDrawConsole;			// is the console drawn?						  default: true
	};

	class _CInterface								// Provides functions that work with the interface (console and legend)
	{
	public:
		void AddCon		(char* cdata, ...);				// Adds data to the console in bottom right
		void AddLegend	(char* cmd, char* description);	// Adds a command to the legend, rendered in the bottom left upon holding F1
		void DrawText2D (char* text, int x, int y, COLOR color); // draws 2d text on screen with shadow
		_CSettings settings;							// structure containing some console/legend-related settings
		LPD3DXFONT DXFont;								// main font object (for drawing 2d text)
	};

	class _CD3D
	{
	public:
		void __stdcall CustomSetTransform(D3DTRANSFORMSTATETYPE State, CONST D3DMATRIX* pMatrix);
	};

	class _CGame									// Provides misc functions that work with the game itself
	{
	public:
		bool			GetCarLockedStatus(unsigned long dwCarPtr);
		BYTE			GetWeekday(); 
		D3DVECTOR		GetActorPos(unsigned long dwAcPtr);
		D3DVECTOR		GetVehiclePos(unsigned long dwCarPtr);
		D3DVECTOR		GetVehicleSpeed(unsigned long dwCarPtr);
		D3DVECTOR		GetVehicleSpin(unsigned long dwCarPtr);
		float			GetActorArmour(unsigned long dwAcPtr);
		float			GetActorHealth(unsigned long dwAcPtr);
		float			GetActorZAngle(unsigned long dwAcPtr);
		float			GetGamespeed();
		float			GetGravity();
		float			GetVehicleHealth(unsigned long dwCarPtr);
		int				GetWeather();
		unsigned long	CreateObject(int iObjectID, float fX, float fY, float fZ);
		unsigned long	CreatePickup(int iObjectID, float fX, float fY, float fZ);
		unsigned long	CreatePlayer(int iModelID, float fX, float fY, float fZ); // create_player(0,x,y,z,&ped)
		unsigned long	GetActor(int iActorID);
		unsigned long	GetLastDrivenVehicle(unsigned long dwAcPtr);
		unsigned long	GetMoney();
		unsigned long	GetVehicle(int iVehicleID);
		unsigned long	GetVehicleDriver(unsigned long dwCarPtr);
		void			DestroyObject(unsigned long dwObject);
		void			DestroyPickup(unsigned long dwPickup);
		void			FadeIn(int iFadeTime, int iR, int iG, int iB);
		void			FadeOut(int iFadeTime, int iR, int iG, int iB);
		void			LoadModel(int iObjectID);
		void			PointCamera(float fX, float fY, float fZ);
		void			SetActorArmour(unsigned long dwAcPtr, float fArmour);
		void			SetActorHealth(unsigned long dwAcPtr, float fHealth);
		void			SetActorZAngle(unsigned long dwAcPtr, float fZAngle);
		void			SetCameraPosition(float fX, float fY, float fZ);
		void			SetCarLockedStatus(unsigned long dwCarPtr, bool bLocked);
		void			SetGamespeed(float fSpeed);
		void			SetGravity(float fGravity);
		void			SetMoney(unsigned long dwAmount);
		void			SetTiresPoppedStatus(unsigned long dwCarPtr, bool bPopped);
		void			SetVehicleHealth(unsigned long dwCarPtr, float fHealth);
		void			SetVehiclePos(unsigned long dwCarPtr, float fX, float fY, float fZ);
		void			SetVehicleSpin(unsigned long dwCarPtr, float fXSpin, float fYSpin, float fZSpin);
		void			SetWeather(int iWeather);
		void			SetWeekday(BYTE bWeekDay);
		void			SetWidescreenStatus(bool bStatus);
		void			UnloadModel(int iObjectID);
		ExecuteOpcode	(const SCRIPT* command, ...);		// Executes an opcode through the scripting engine (credits to Spookie & op9080)
	};

	_CInterface		__declspec(dllexport) *GetInterfaceClass();
	_CInterface*	CInterface = GetInterfaceClass(); // grab core's interface class
	_CGame			__declspec(dllexport) *GetGameClass();
	_CGame*			CGame = GetGameClass(); // grab core's game class
	_CD3D			__declspec(dllexport) *GetD3DClass();
	_CD3D*			CD3D = GetD3DClass(); // grab core's D3D class


// DIRECTX FORWARDS (the core calls these when certain dx events occur)
// ------------------------------------------
	void			__stdcall			CustomPresent	(IDirect3DDevice9* pDevice);
	void			__stdcall			CustomInitialize	(IDirect3DDevice9* pDevice, D3DPRESENT_PARAMETERS* pPresentationParameters);
	void			__stdcall			CustomDrawIndexedPrimitive(D3DPRIMITIVETYPE Type, INT BaseVertexIndex, UINT MinVertexIndex, UINT NumVertices, UINT startIndex, UINT primCount);


// KEEP MULTIHACK 1.1 SUPPORT!! (old non-class functions)
// ------------------------------------------
	void			__declspec(dllexport)	AddCommandToLegend(char* cmd, char* description);
	void			__declspec(dllexport)	AddConsole(char* cdata, int iR, int iG, int iB); 
	int				__declspec(dllexport)	ScriptCommand(const SCRIPT* command, ...);


// SCRIPTING ENGINE STUFF
// ------------------------------------------


const SCRIPT wait                     = { 0x0001, "i"		};	// DURATION
const SCRIPT force_weather            = { 0x01B5, "i"		};	// WEATHER
const SCRIPT set_gamespeed            = { 0x015d, "f"		};	// GAMESPEED
const SCRIPT create_car               = { 0x00a5, "ifffv"	};	// #MODEL, X, Y, Z, VAR
const SCRIPT destroy_car              = { 0x00a6, "v"		};	// VAR(CAR)
const SCRIPT request_model            = { 0x0247, "i"		};	// #MODEL
const SCRIPT load_requested_models    = { 0x038b, ""		};	// N/A
const SCRIPT create_actor             = { 0x009a, "iifffv"	};	// PEDTYPE, #MODEL, X, Y, Z, VAR(PED)
const SCRIPT create_fire_on_actor     = { 0x0326, "vv"		};	// VAR(PED), VAR(FIRE)
const SCRIPT actor_enter_car          = { 0x05CB, "vvi"		};	// VAR(PED), VAR(CAR), TIME(-1)
const SCRIPT set_car_driver_behaviour = { 0x00af, "vi"		};	// VAR(CAR), BEHAVIOR
const SCRIPT give_actor_weapon        = { 0x01b2, "vii"		};	// VAR(PED), WEAPON, AMMO
const SCRIPT create_actor_in_car      = { 0x0129, "viiv"	};	// VAR(CAR), PED_TYPE, MODEL_ID, VAR(PED)
const SCRIPT get_actor_weapon         = { 0x0470, "vv"		};	// VAR(PED), VAR(WEAPON)
const SCRIPT set_actor_weapon_ammo    = { 0x017b, "vvi"		};	// VAR(PED), VAR(WEAPON) AMMO
const SCRIPT set_actor_armed_weapon   = { 0x01b9, "vv"		};	// VAR(PED), VAR(WEAPON)
const SCRIPT set_car_driver_psycho    = { 0x00a8, "v"		};	// VAR(CAR)
const SCRIPT put_actor_in_car         = { 0x036a, "vv"		};	// VAR(ACTOR, VAR(CAR)
const SCRIPT create_player            = { 0x0053, "ifffv"	};	// 0, X, Y, Z, VAR(PED) 
const SCRIPT get_car_coords			  = { 0x00aa, "vfff"	};	// var(car), x,y,z
const SCRIPT create_object			  = { 0x0107, "ifffv"	};  // #object, X, Y, Z, VAR(OBJECT)
const SCRIPT destroy_object			  = { 0x0108, "v"		};	// VAR(OBJECT)
const SCRIPT create_pickup			  = { 0x0213, "iifffv"	};	// #model, 0, x, y, z, VAR(pickup)
const SCRIPT destroy_pickup			  = { 0x0215, "v"		};	// VAR(pickup)
const SCRIPT is_model_loaded		  = { 0x0248, "i"		};	// #model
const SCRIPT set_fade_color			  = { 0x0169, "iii"		};	// R, G, B
const SCRIPT fade					  = { 0x016a, "ii"		};	// time, 0=out/1=in
const SCRIPT toggle_widescreen		  = { 0x02a3, "i"		};	// 0=off, 1=on
const SCRIPT release_model			  = { 0x0249, "i"		};	// #model
const SCRIPT point_camera			  = { 0x0160, "fffi"	};	// X,Y,Z,1=pan/2=cut
const SCRIPT set_camera_position	  = { 0x015f, "ffffff"	};	// X,Y,Z,0,0,0

#endif